@@ -30,7 +30,9 @@
 
 #include <libxml/SAX.h>
 #include <libxml/xmlmemory.h>
-#include <pango/pangoft2.h>
+#include <pango/pango.h>
+#include <libart_lgpl/art_rect.h>
+#include <glib/gslist.h>
 
 G_BEGIN_DECLS
 
@@ -58,16 +60,19 @@ struct RsvgHandle {
 	gpointer user_data;
 	GDestroyNotify user_data_destroy;
 	GdkPixbuf *pixbuf;
-	
+	ArtIRect bbox;
+
 	/* stack; there is a state for each element */
-	RsvgState *state;
-	int n_state;
-	int n_state_max;
+
+	GSList * state;
 	
 	RsvgDefs *defs;
 	guint in_defs;
+	guint nest_level;
 	void *current_defs_group;
 
+	guint in_switch;
+
 	GHashTable *css_props;
 	
 	/* not a handler stack. each nested handler keeps
@@ -84,14 +89,19 @@ struct RsvgHandle {
 	
 	int width;
 	int height;
-	double dpi;
+	double dpi_x;
+	double dpi_y;
 	
 	GString * title;
 	GString * desc;
+	
+	gchar * base_uri;
 
 	void * currentfilter;
 	void * currentsubfilter;
 
+	GMemChunk * state_allocator;
+
 	/* virtual fns */
 	gboolean (* write) (RsvgHandle    *handle,
 						const guchar  *buf,
@@ -151,7 +161,7 @@ rsvg_property_bag_new (const xmlChar **atts);
 void
 rsvg_property_bag_free (RsvgPropertyBag *bag);
 
-const char *
+G_CONST_RETURN char *
 rsvg_property_bag_lookup (RsvgPropertyBag *bag, const char * key);
 
 guint
@@ -161,8 +171,22 @@ GdkPixbuf *
 rsvg_pixbuf_from_data_with_size_data (const guchar * buff,
 									  size_t len,
 									  struct RsvgSizeCallbackData * data,
+									  const char * base_uri,
 									  GError ** error);
 
+G_CONST_RETURN char *
+rsvg_handle_get_base_uri (RsvgHandle *handle);
+
+void rsvg_handle_set_base_uri (RsvgHandle *handle,
+							   const char *base_uri);
+
+gboolean 
+rsvg_eval_switch_attributes (RsvgPropertyBag *atts, gboolean * p_has_cond);
+
+GdkPixbuf *
+_rsvg_pixbuf_new_cleared (GdkColorspace colorspace, gboolean has_alpha, int bits_per_sample,
+						  int width, int height);
+
 G_END_DECLS
 
 #endif
