@@ -23,6 +23,7 @@
    Author: Raph Levien <raph@artofcode.com>
 */
 #include <string.h>
+#include <math.h>
 
 #include "rsvg.h"
 #include "rsvg-private.h"
@@ -34,6 +35,7 @@
 
 #include <libart_lgpl/art_rgba.h>
 #include <libart_lgpl/art_affine.h>
+#include <libart_lgpl/art_svp_ops.h>
 
 #define RSVG_DEFAULT_FONT "Times New Roman"
 
@@ -54,7 +56,13 @@ rsvg_state_current_color (RsvgState * cur_state, RsvgState * parent_state)
 gdouble
 rsvg_viewport_percentage (gdouble width, gdouble height)
 {
-	return ((width * width) + (height * height)) / M_SQRT2;
+	return sqrt(width * height);
+}
+
+gdouble
+rsvg_dpi_percentage (RsvgHandle * ctx)
+{
+	return sqrt(ctx->dpi_x * ctx->dpi_y);
 }
 
 void
@@ -76,6 +84,7 @@ rsvg_state_init (RsvgState *state)
 	state->join = ART_PATH_STROKE_JOIN_MITER;
 	state->stop_opacity = 0xff;
 	state->fill_rule = FILL_RULE_NONZERO;
+	state->clip_rule = FILL_RULE_NONZERO;
 	state->backgroundnew = FALSE;
 	state->save_pixbuf = NULL;
 
@@ -86,14 +95,21 @@ rsvg_state_init (RsvgState *state)
 	state->font_weight  = PANGO_WEIGHT_NORMAL;
 	state->font_stretch = PANGO_STRETCH_NORMAL;
 	state->text_dir     = PANGO_DIRECTION_LTR;
+	state->unicode_bidi = UNICODE_BIDI_NORMAL;
 	state->text_anchor  = TEXT_ANCHOR_START;
 	state->visible      = TRUE;
+	state->cond_true    = TRUE;
 	state->filter       = NULL;
+	state->clip_path_ref= NULL;
+	state->startMarker  = NULL;
+	state->middleMarker = NULL;
+	state->endMarker    = NULL;
 
 	state->has_current_color = FALSE;
 	state->has_fill_server = FALSE;
 	state->has_fill_opacity = FALSE;
 	state->has_fill_rule = FALSE;
+	state->has_clip_rule = FALSE;
 	state->has_stroke_server = FALSE;
 	state->has_stroke_opacity = FALSE;
 	state->has_stroke_width = FALSE;
@@ -102,6 +118,7 @@ rsvg_state_init (RsvgState *state)
 	state->has_join = FALSE;
 	state->has_dash = FALSE;
 	state->has_visible = FALSE;
+	state->has_cond = FALSE;
 	state->has_stop_color = FALSE;
 	state->has_stop_opacity = FALSE;
 	state->has_font_size = FALSE;
@@ -113,7 +130,14 @@ rsvg_state_init (RsvgState *state)
 	state->has_font_stretch = FALSE;
 	state->has_font_decor = FALSE;
 	state->has_text_dir = FALSE;
+	state->has_unicode_bidi = FALSE;
 	state->has_text_anchor = FALSE;
+	state->has_startMarker = FALSE;
+	state->has_middleMarker = FALSE;
+	state->has_endMarker = FALSE;
+
+	state->clippath = NULL;
+	state->clip_path_loaded = FALSE;
 }
 
 void
@@ -153,6 +177,8 @@ rsvg_state_reinherit (RsvgState *dst, const RsvgState *src)
 		dst->fill_opacity = src->fill_opacity;
 	if (!dst->has_fill_rule)
 		dst->fill_rule = src->fill_rule;
+	if (!dst->has_clip_rule)
+		dst->clip_rule = src->clip_rule;
 	if (!dst->has_stroke_server)
 		{
 			rsvg_paint_server_ref (src->stroke);
@@ -175,6 +201,8 @@ rsvg_state_reinherit (RsvgState *dst, const RsvgState *src)
 		dst->stop_opacity = src->stop_opacity;
 	if (!dst->has_visible)
 		dst->visible = src->visible;
+	if (!dst->has_cond)
+		dst->cond_true = src->cond_true;
 	if (!dst->has_font_size)
 		dst->font_size = src->font_size;
 	if (!dst->has_font_style)
@@ -189,8 +217,16 @@ rsvg_state_reinherit (RsvgState *dst, const RsvgState *src)
 		dst->font_decor = src->font_decor;
 	if (!dst->has_text_dir)
 		dst->text_dir = src->text_dir;
+	if (!dst->has_unicode_bidi)
+		dst->unicode_bidi = src->unicode_bidi;
 	if (!dst->has_text_anchor)
 		dst->text_anchor = src->text_anchor;
+	if (!dst->has_startMarker)
+		dst->startMarker = src->startMarker;
+	if (!dst->has_middleMarker)
+		dst->middleMarker = src->middleMarker;
+	if (!dst->has_endMarker)
+		dst->endMarker = src->endMarker;
 
 	if (!dst->has_font_family) {
 		g_free(dst->font_family); /* font_family is always set to something */
@@ -208,6 +244,8 @@ rsvg_state_reinherit (RsvgState *dst, const RsvgState *src)
 				dst->dash.dash[i] = src->dash.dash[i];
 		}
 	art_affine_multiply (dst->affine, dst->personal_affine, src->affine); 
+
+	dst->clippath = src->clippath;
 }
 
 void
@@ -227,6 +265,8 @@ rsvg_state_dominate (RsvgState *dst, const RsvgState *src)
 		dst->fill_opacity = src->fill_opacity;
 	if (!dst->has_fill_rule || src->has_fill_rule)
 		dst->fill_rule = src->fill_rule;
+	if (!dst->has_clip_rule || src->has_clip_rule)
+		dst->clip_rule = src->clip_rule;
 	if (!dst->has_stroke_server || src->has_stroke_server)
 		{
 			rsvg_paint_server_ref (src->stroke);
@@ -247,8 +287,8 @@ rsvg_state_dominate (RsvgState *dst, const RsvgState *src)
 		dst->stop_color = src->stop_color;				
 	if (!dst->has_stop_opacity || src->has_stop_opacity)
 		dst->stop_opacity = src->stop_opacity;
-	if (!dst->has_visible || src->has_visible)
-		dst->visible = src->visible;
+	if (!dst->has_cond || src->has_cond)
+		dst->cond_true = src->cond_true;
 	if (!dst->has_font_size || src->has_font_size)
 		dst->font_size = src->font_size;
 	if (!dst->has_font_style || src->has_font_style)
@@ -263,8 +303,16 @@ rsvg_state_dominate (RsvgState *dst, const RsvgState *src)
 		dst->font_decor = src->font_decor;
 	if (!dst->has_text_dir || src->has_text_dir)
 		dst->text_dir = src->text_dir;
+	if (!dst->has_unicode_bidi || src->has_unicode_bidi)
+		dst->unicode_bidi = src->unicode_bidi;
 	if (!dst->has_text_anchor || src->has_text_anchor)
 		dst->text_anchor = src->text_anchor;
+	if (!dst->has_startMarker || !src->has_startMarker)
+		dst->startMarker = src->startMarker;
+	if (!dst->has_middleMarker || !src->middleMarker)
+		dst->middleMarker = src->middleMarker;
+	if (!dst->has_endMarker || !src->endMarker)
+		dst->endMarker = src->endMarker;
 
 	if (!dst->has_font_family || src->has_font_family) {
 		g_free(dst->font_family); /* font_family is always set to something */
@@ -276,7 +324,7 @@ rsvg_state_dominate (RsvgState *dst, const RsvgState *src)
 			g_free(dst->lang); 
 		dst->lang = g_strdup (src->lang);
 	}
-
+	
 	if (src->dash.n_dash > 0 && (!dst->has_dash || src->has_dash))
 		{
 			if(dst->has_dash)
@@ -287,6 +335,7 @@ rsvg_state_dominate (RsvgState *dst, const RsvgState *src)
 				dst->dash.dash[i] = src->dash.dash[i];
 		}
 	art_affine_multiply (dst->affine, dst->personal_affine, src->affine); 
+	dst->clippath = src->clippath;
 }
 
 void
@@ -366,6 +415,10 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 		}
 	else if (rsvg_css_param_match (str, "mask"))
 		state->mask = rsvg_mask_parse(ctx->defs, str + arg_off);
+	else if (rsvg_css_param_match (str, "clip-path"))
+		{
+			state->clip_path_ref = rsvg_clip_path_parse(ctx->defs, str + arg_off);
+		}
 	else if (rsvg_css_param_match (str, "enable-background"))
 		{
 			if (!strcmp (str + arg_off, "new"))
@@ -422,6 +475,16 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 			else
 				state->has_fill_rule = FALSE;
 		}
+	else if (rsvg_css_param_match (str, "clip-rule"))
+		{
+			state->has_clip_rule = TRUE;
+			if (!strcmp (str + arg_off, "nonzero"))
+				state->clip_rule = FILL_RULE_NONZERO;
+			else if (!strcmp (str + arg_off, "evenodd"))
+				state->clip_rule = FILL_RULE_EVENODD;
+			else
+				state->has_clip_rule = FALSE;
+		}
 	else if (rsvg_css_param_match (str, "stroke"))
 		{
 			RsvgPaintServer * stroke = state->stroke;
@@ -438,7 +501,7 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 		}
 	else if (rsvg_css_param_match (str, "stroke-width"))
 		{
-			state->stroke_width = rsvg_css_parse_normalized_length (str + arg_off, ctx->dpi, 
+			state->stroke_width = rsvg_css_parse_normalized_length (str + arg_off, ctx->dpi_x, 
 																	(gdouble)ctx->height, state->font_size);
 			state->has_stroke_width = TRUE;
 		}
@@ -452,7 +515,7 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 			else if (!strcmp (str + arg_off, "square"))
 				state->cap = ART_PATH_STROKE_CAP_SQUARE;
 			else
-				g_warning ("unknown line cap style %s", str + arg_off);
+				g_warning (_("unknown line cap style %s\n"), str + arg_off);
 		}
 	else if (rsvg_css_param_match (str, "stroke-opacity"))
 		{
@@ -469,12 +532,12 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 			else if (!strcmp (str + arg_off, "bevel"))
 				state->join = ART_PATH_STROKE_JOIN_BEVEL;
 			else
-				g_warning ("unknown line join style %s", str + arg_off);
+				g_warning (_("unknown line join style %s\n"), str + arg_off);
 		}
 	else if (rsvg_css_param_match (str, "font-size"))
 		{
-			state->font_size = rsvg_css_parse_normalized_length (str + arg_off, ctx->dpi, 
-																 (gdouble)ctx->height, state->font_size);
+			state->font_size =  rsvg_css_parse_normalized_length (str + arg_off, ctx->dpi_y, 
+																  (gdouble)ctx->height, state->font_size);
 			state->has_font_size = TRUE;
 		}
 	else if (rsvg_css_param_match (str, "font-family"))
@@ -534,25 +597,53 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 					state->has_font_decor = TRUE;
 				}
 		}
-	else if (rsvg_css_param_match (str, "writing-mode"))
+	else if (rsvg_css_param_match (str, "direction"))
 		{
 			state->has_text_dir = TRUE;
-			/* lr-tb | rl-tb | tb-rl | lr | rl | tb | inherit */
 			if (!strcmp (str + arg_off, "inherit"))
 				{
 					state->text_dir = (parent_state ? parent_state->text_dir : PANGO_DIRECTION_LTR);
 					state->has_text_dir = FALSE;
 				}
+			else if (!strcmp (str + arg_off, "rtl"))
+				state->text_dir = PANGO_DIRECTION_RTL;
+			else /* ltr */
+				state->text_dir = PANGO_DIRECTION_LTR;
+		}
+	else if (rsvg_css_param_match (str, "unicode-bidi"))
+		{
+			state->has_unicode_bidi = TRUE;
+			if (!strcmp (str + arg_off, "inherit"))
+				{
+					state->unicode_bidi = (parent_state ? parent_state->unicode_bidi : PANGO_DIRECTION_LTR);
+					state->has_unicode_bidi = FALSE;
+				}
+			else if (!strcmp (str + arg_off, "embed"))
+				state->unicode_bidi = UNICODE_BIDI_EMBED;
+			else if (!strcmp (str + arg_off, "bidi-override"))
+				state->unicode_bidi = UNICODE_BIDI_OVERRIDE;
+			else /* normal */
+				state->unicode_bidi = UNICODE_BIDI_NORMAL;
+		}
+	else if (rsvg_css_param_match (str, "writing-mode"))
+		{
+			/* TODO: these aren't quite right... */
+
+			state->has_text_dir = TRUE;
+			if (!strcmp (str + arg_off, "inherit")) {
+				state->text_dir = parent_state->text_dir;
+				state->has_text_dir = FALSE;
+			}
+			else if (!strcmp (str + arg_off, "lr-tb") ||
+					 !strcmp (str + arg_off, "tb"))
+				state->text_dir = PANGO_DIRECTION_TTB_LTR;
 			else if (!strcmp (str + arg_off, "rl"))
 				state->text_dir = PANGO_DIRECTION_RTL;
 			else if (!strcmp (str + arg_off, "tb-rl") || 
-					 !strcmp (str + arg_off, "rl-tb")) /* not sure of tb-rl vs. rl-tb */
+					 !strcmp (str + arg_off, "rl-tb"))
 				state->text_dir = PANGO_DIRECTION_TTB_RTL;
-			else if (!strcmp (str + arg_off, "lr-tb"))
-				state->text_dir = PANGO_DIRECTION_TTB_LTR;
 			else
 				state->text_dir = PANGO_DIRECTION_LTR;
-
 		}
 	else if (rsvg_css_param_match (str, "text-anchor"))
 		{
@@ -589,6 +680,21 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 					state->stop_opacity = rsvg_css_parse_opacity (str + arg_off);
 				}
 		}
+	else if (rsvg_css_param_match (str, "marker-start"))
+		{
+			state->startMarker = rsvg_marker_parse(ctx->defs, str + arg_off);
+			state->has_startMarker = TRUE;
+		}
+	else if (rsvg_css_param_match (str, "marker-mid"))
+		{
+			state->middleMarker = rsvg_marker_parse(ctx->defs, str + arg_off);
+			state->has_middleMarker = TRUE;
+		}
+	else if (rsvg_css_param_match (str, "marker-end"))
+		{
+			state->endMarker = rsvg_marker_parse(ctx->defs, str + arg_off);
+			state->has_endMarker = TRUE;
+		}
 	else if (rsvg_css_param_match (str, "stroke-miterlimit"))
 		{
 			state->has_miter_limit = TRUE;
@@ -597,7 +703,7 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 	else if (rsvg_css_param_match (str, "stroke-dashoffset"))
 		{
 			state->has_dash = TRUE;
-			state->dash.offset = rsvg_css_parse_normalized_length (str + arg_off, ctx->dpi, 
+			state->dash.offset = rsvg_css_parse_normalized_length (str + arg_off, rsvg_dpi_percentage (ctx), 
 																   rsvg_viewport_percentage((gdouble)ctx->width, (gdouble)ctx->height), state->font_size);
 			if (state->dash.offset < 0.)
 				state->dash.offset = 0.;
@@ -669,37 +775,56 @@ void
 rsvg_parse_style_pairs (RsvgHandle *ctx, RsvgState *state, 
 						RsvgPropertyBag *atts)
 {
-			rsvg_lookup_parse_style_pair (ctx, state, "a:adobe-blending-mode", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "color", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "display", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "enable-background", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "fill", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "fill-opacity", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "fill-rule", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "font-family", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "font-size", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "font-stretch", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "font-style", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "font-variant", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "font-weight", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "opacity", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "mask", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "filter", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "stop-color", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "stop-opacity", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "stroke", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "stroke-dasharray", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "stroke-dashoffset", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "stroke-linecap", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "stroke-linejoin", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "stroke-miterlimit", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "stroke-opacity", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "stroke-width", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "text-anchor", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "text-decoration", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "visibility", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "writing-mode", atts);
-			rsvg_lookup_parse_style_pair (ctx, state, "xml:lang", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "a:adobe-blending-mode", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "clip-path", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "clip-rule", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "color", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "direction", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "display", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "enable-background", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "fill", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "fill-opacity", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "fill-rule", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "filter", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "font-family", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "font-size", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "font-stretch", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "font-style", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "font-variant", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "font-weight", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "marker-end", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "mask", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "marker-mid", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "marker-start", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "opacity", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "stop-color", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "stop-opacity", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "stroke", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "stroke-dasharray", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "stroke-dashoffset", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "stroke-linecap", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "stroke-linejoin", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "stroke-miterlimit", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "stroke-opacity", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "stroke-width", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "text-anchor", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "text-decoration", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "unicode-bidi", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "visibility", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "writing-mode", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "xml:lang", atts);
+	
+	{
+		/* TODO: this conditional behavior isn't quite correct, and i'm not sure it should reside here */
+		gboolean cond_true, has_cond;
+
+		cond_true = rsvg_eval_switch_attributes (atts, &has_cond);
+
+		if (has_cond) {
+			state->cond_true = cond_true;
+			state->has_cond = TRUE;
+		}
+	}
 }
 
 /* Split a CSS2 style into individual style arguments, setting attributes
@@ -830,7 +955,7 @@ ccss_error (CRDocHandler *a_handler)
 {
 	/* yup, like i care about CSS parsing errors ;-)
 	   ignore, chug along */
-	g_warning ("CSS parsing error\n");
+	g_warning (_("CSS parsing error\n"));
 }
 
 static void
@@ -838,7 +963,7 @@ ccss_unrecoverable_error (CRDocHandler *a_handler)
 {
 	/* yup, like i care about CSS parsing errors ;-)
 	   ignore, chug along */
-	g_warning ("CSS unrecoverable error\n");
+	g_warning (_("CSS unrecoverable error\n"));
 }
 
 static void
@@ -883,7 +1008,7 @@ rsvg_real_parse_cssbuffer (RsvgHandle *ctx, const char * buff, size_t buflen)
     
 	if (status != CR_OK)
         {
-			g_warning (_("Error setting CSS SAC handler"));
+			g_warning (_("Error setting CSS SAC handler\n"));
 			cr_parser_destroy (parser);
 			return;
         }        
@@ -1068,10 +1193,20 @@ rsvg_parse_transform (double dst[6], const char *src)
 				}
 			else if (!strcmp (keyword, "rotate"))
 				{
-					if (n_args != 1)
+					if (n_args == 1) {
+						art_affine_rotate (tmp_affine, args[0]);
+						art_affine_multiply (dst, tmp_affine, dst);
+					} else if (n_args == 3) {
+						art_affine_translate (tmp_affine, args[1], args[2]);
+						art_affine_multiply (dst, tmp_affine, dst);
+					
+						art_affine_rotate (tmp_affine, args[0]);
+						art_affine_multiply (dst, tmp_affine, dst);
+						
+						art_affine_translate (tmp_affine, -args[1], -args[2]);
+						art_affine_multiply (dst, tmp_affine, dst);
+					} else
 						return FALSE;
-					art_affine_rotate (tmp_affine, args[0]);
-					art_affine_multiply (dst, tmp_affine, dst);
 				}
 			else if (!strcmp (keyword, "skewX"))
 				{
@@ -1123,7 +1258,7 @@ rsvg_lookup_apply_css_style (RsvgHandle *ctx, const char * target)
 	
 	if (value != NULL)
 		{
-			rsvg_parse_style (ctx, &ctx->state[ctx->n_state - 1],
+			rsvg_parse_style (ctx, rsvg_state_current(ctx),
 							  value);
 			return TRUE;
 		}
@@ -1261,15 +1396,22 @@ rsvg_push_discrete_layer (RsvgHandle *ctx)
 	art_u8 *pixels;
 	int width, height, rowstride;
 
-	state = &ctx->state[ctx->n_state - 1];
+	state = rsvg_state_current(ctx);
 	pixbuf = ctx->pixbuf;
-	
+
+	rsvg_state_clip_path_assure(ctx);
+
 	if (state->filter == NULL && state->opacity == 0xFF && 
 		!state->backgroundnew && state->mask == NULL && !state->adobe_blend)
 		return;
-
-	state->save_pixbuf = pixbuf;
 	
+	state->save_pixbuf = pixbuf;
+	state->underbbox = ctx->bbox;	
+	ctx->bbox.x0 = 0;
+	ctx->bbox.x1 = 0;
+	ctx->bbox.y0 = 0;
+	ctx->bbox.y1 = 0;
+
 	if (pixbuf == NULL)
 		{
 			/* FIXME: What warning/GError here? */
@@ -1278,7 +1420,7 @@ rsvg_push_discrete_layer (RsvgHandle *ctx)
 
 	if (!gdk_pixbuf_get_has_alpha (pixbuf))
     {
-		g_warning (_("push/pop transparency group on non-alpha buffer nyi"));
+		g_warning (_("push/pop transparency group on non-alpha buffer nyi\n"));
 		return;
     }
 	
@@ -1320,7 +1462,7 @@ rsvg_use_opacity (RsvgHandle *ctx, int opacity,
 	
 	if (!gdk_pixbuf_get_has_alpha (nos))
 		{
-			g_warning (_("push/pop transparency group on non-alpha buffer nyi"));
+			g_warning (_("push/pop transparency group on non-alpha buffer nyi\n"));
 			return;
 		}
 	
@@ -1330,10 +1472,13 @@ rsvg_use_opacity (RsvgHandle *ctx, int opacity,
 	
 	tos_pixels = gdk_pixbuf_get_pixels (tos);
 	nos_pixels = gdk_pixbuf_get_pixels (nos);
+
+	tos_pixels += rowstride * MAX(ctx->bbox.y0, 0);
+	nos_pixels += rowstride * MAX(ctx->bbox.y0, 0);
 	
-	for (y = 0; y < height; y++)
+	for (y = MAX(ctx->bbox.y0, 0); y < MIN(ctx->bbox.y1 + 1, height); y++)
 		{
-			for (x = 0; x < width; x++)
+			for (x = MAX(ctx->bbox.x0, 0); x < MIN(ctx->bbox.x1 + 1, width); x++)
 				{
 					art_u8 r, g, b, a;
 					a = tos_pixels[4 * x + 3];
@@ -1384,9 +1529,8 @@ rsvg_compile_bg(RsvgHandle *ctx, RsvgState *topstate)
 
 	lastintermediate = gdk_pixbuf_copy(topstate->save_pixbuf);
 			
-	for (i = ctx->n_state - 1; i >= 0; i--)
+	for (i = 0; (state = g_slist_nth_data(ctx->state, i)) != NULL; i++)
 		{
-			state = &ctx->state[i];
 			if (state == topstate)
 				{
 					foundstate = 1;
@@ -1423,10 +1567,6 @@ rsvg_composite_layer(RsvgHandle *ctx, RsvgState *state, GdkPixbuf *tos, GdkPixbu
 
 	intermediate = NULL;
 
-	if (state->filter == NULL && state->opacity == 0xFF && 
-		!state->backgroundnew && state->mask == NULL && !state->adobe_blend)
-		return;
-
 	operationsleft = 0;
 	
 	if (opacity != 0xFF)
@@ -1478,7 +1618,7 @@ rsvg_composite_layer(RsvgHandle *ctx, RsvgState *state, GdkPixbuf *tos, GdkPixbu
 	if (adobe_blend)
 		{
 			out = get_next_out(&operationsleft, in, tos, nos, intermediate);
-			rsvg_filter_adobe_blend (adobe_blend, in, insidebg, out);
+			rsvg_filter_adobe_blend (adobe_blend, in, insidebg, out, ctx);
 			in = out;
 		}
 
@@ -1520,6 +1660,7 @@ rsvg_pop_discrete_layer(RsvgHandle *ctx)
 	
 	g_object_unref (tos);
 	ctx->pixbuf = nos;
+	art_irect_union(&ctx->bbox, &ctx->bbox, &state->underbbox);
 }
 
 gboolean
@@ -1531,24 +1672,20 @@ rsvg_needs_discrete_layer(RsvgState *state)
 RsvgState * 
 rsvg_state_current (RsvgHandle *ctx)
 {
-	if (ctx->n_state > 0)
-		return &ctx->state[ctx->n_state - 1];
-	return NULL;
+	return g_slist_nth_data(ctx->state, 0);
 }
 
 RsvgState *
 rsvg_state_parent (RsvgHandle *ctx)
 {
-	if (ctx->n_state > 1)
-		return &ctx->state[ctx->n_state - 2];
-	return NULL;
+	return g_slist_nth_data(ctx->state, 1);
 }
 
 double
 rsvg_state_current_font_size (RsvgHandle *ctx)
 {
-	if (ctx->n_state > 0)
-		return ctx->state[ctx->n_state - 1].font_size;
+	if (rsvg_state_current(ctx) != NULL)
+		return rsvg_state_current(ctx)->font_size;
 	else
 		return 12.0;
 }
@@ -1578,7 +1715,7 @@ rsvg_property_bag_free (RsvgPropertyBag *bag)
 	g_free (bag);
 }
 
-const char *
+G_CONST_RETURN char *
 rsvg_property_bag_lookup (RsvgPropertyBag *bag, const char * key)
 {
 	return (const char *)g_hash_table_lookup (bag->props, (gconstpointer)key);
@@ -1589,3 +1726,96 @@ rsvg_property_bag_size (RsvgPropertyBag *bag)
 {
 	return g_hash_table_size (bag->props);
 }
+
+void 
+rsvg_state_clip_path_assure(RsvgHandle * ctx)
+{
+	ArtSVP * tmppath;
+	ArtSVP * tmppath2;
+	RsvgState * state;
+
+	state = rsvg_state_current(ctx);
+
+	if (state->clip_path_ref && !state->clip_path_loaded)
+		{
+			rsvg_state_push(ctx);
+
+			tmppath = rsvg_clip_path_render (state->clip_path_ref, ctx);
+
+			rsvg_state_pop(ctx);
+
+			state->clip_path_loaded = TRUE;
+		}
+	else
+		return;
+
+	if (state->clippath != NULL && tmppath != NULL)
+		{
+			tmppath2 = art_svp_intersect(tmppath, state->clippath);
+			art_free(tmppath);
+			state->clippath = tmppath2;
+		}
+	else
+		state->clippath = tmppath;
+}
+
+void
+rsvg_state_push(RsvgHandle * ctx)
+{
+	RsvgState * data;
+	RsvgState * baseon;
+
+	baseon = (RsvgState *)g_slist_nth_data(ctx->state, 0);
+	data = g_chunk_new(RsvgState, ctx->state_allocator);
+
+	if (baseon)
+		rsvg_state_inherit(data, baseon);
+	else
+		rsvg_state_init(data);
+
+	ctx->state = g_slist_prepend(ctx->state, data);
+}
+
+void
+rsvg_state_pop(RsvgHandle * ctx)
+{
+	RsvgState * toremove = g_slist_nth_data(ctx->state, 0);
+	rsvg_state_finalize (toremove);
+	ctx->state = g_slist_remove(ctx->state, toremove);	
+	g_mem_chunk_free(ctx->state_allocator, toremove);
+}
+
+void
+rsvg_state_reinherit_top(RsvgHandle * ctx, RsvgState * state, int dominate)
+{
+	double tempaffine[6];
+	gint i;
+	RsvgState * baseon;
+
+	baseon = rsvg_state_parent(ctx);
+
+	for (i = 0; i < 6; i++)
+		{
+			 tempaffine[i] = rsvg_state_current(ctx)->affine[i];
+		}	
+
+	/* combine state definitions */
+	rsvg_state_clone (rsvg_state_current(ctx), state);
+
+	if (baseon)
+		{
+			/*This is a special domination mode for patterns, the style
+			  is simply left as is, wheras the transform is totally overridden*/
+			if (dominate == 2)
+				{
+					for (i = 0; i < 6; i++)
+						{
+							rsvg_state_current(ctx)->affine[i] = tempaffine[i];
+						}	
+				}		
+			else if (dominate)
+				rsvg_state_dominate(rsvg_state_current(ctx), baseon);
+			else
+				rsvg_state_reinherit(rsvg_state_current(ctx), baseon);
+		}
+}
