@@ -32,7 +32,7 @@
 #include <libxml/SAX.h>
 #include <libart_lgpl/art_svp_vpath_stroke.h>
 #include <libart_lgpl/art_vpath_dash.h>
-#include <pango/pangoft2.h>
+#include <pango/pango.h>
 
 G_BEGIN_DECLS
 
@@ -56,6 +56,12 @@ enum {
 	FILL_RULE_NONZERO = 1
 };
 
+typedef enum {
+	UNICODE_BIDI_NORMAL = 0,
+	UNICODE_BIDI_EMBED = 1,
+	UNICODE_BIDI_OVERRIDE = 2	
+} UnicodeBidi;
+
 struct _RsvgState {
 	double affine[6];
 	double personal_affine[6];
@@ -68,10 +74,14 @@ struct _RsvgState {
 	gboolean has_fill_opacity;
 	gint fill_rule;	
 	gboolean has_fill_rule;
+	gint clip_rule;	
+	gboolean has_clip_rule;
 
 	RsvgFilter *filter;
 	void *mask;
+	void *clip_path_ref;
 	gboolean backgroundnew;
+	gint adobe_blend;
 
 	RsvgPaintServer *stroke;
 	gboolean has_stroke_server;
@@ -105,6 +115,8 @@ struct _RsvgState {
 	gboolean has_font_decor;
 	PangoDirection text_dir;
 	gboolean has_text_dir;
+	UnicodeBidi unicode_bidi;
+	gboolean has_unicode_bidi;
 	TextAnchor     text_anchor;
 	gboolean has_text_anchor;	
 
@@ -118,13 +130,27 @@ struct _RsvgState {
 	gboolean visible;
 	gboolean has_visible;
 
+	gboolean has_cond;
+	gboolean cond_true;
+
 	ArtVpathDash dash;
 	gboolean has_dash;
 
 	guint32 current_color;
 	gboolean has_current_color;
 
+	RsvgDefVal * startMarker;
+	RsvgDefVal * middleMarker;
+	RsvgDefVal * endMarker;	
+	gboolean has_startMarker;
+	gboolean has_middleMarker;
+	gboolean has_endMarker;	
+
 	GdkPixbuf *save_pixbuf;
+	ArtIRect underbbox;
+
+	ArtSVP * clippath;
+	gboolean clip_path_loaded;
 };
 
 void rsvg_state_init (RsvgState *state);
@@ -146,15 +172,24 @@ void rsvg_parse_style_attrs (RsvgHandle *ctx, RsvgState *state, const char * tag
 							 RsvgPropertyBag *atts);
 
 gdouble rsvg_viewport_percentage (gdouble width, gdouble height);
-void
-rsvg_pop_discrete_layer(RsvgHandle *ctx);
+gdouble rsvg_dpi_percentage (RsvgHandle * ctx);
+
+void rsvg_pop_discrete_layer(RsvgHandle *ctx);
 void rsvg_push_discrete_layer (RsvgHandle *ctx);
+gboolean rsvg_needs_discrete_layer(RsvgState *state);
 gboolean rsvg_parse_transform (double dst[6], const char *src);
 
 RsvgState * rsvg_state_parent (RsvgHandle *ctx);
 RsvgState * rsvg_state_current (RsvgHandle *ctx);
 double rsvg_state_current_font_size (RsvgHandle *ctx);
 
+void rsvg_state_clip_path_assure(RsvgHandle * ctx);
+
+void rsvg_state_pop(RsvgHandle * ctx);
+void rsvg_state_push(RsvgHandle * ctx);
+
+void rsvg_state_reinherit_top(RsvgHandle * ctx, RsvgState * state, int dominate);
+
 G_END_DECLS
 
 #endif /* RSVG_STYLES_H */
