@@ -47,8 +47,10 @@
 /*
  * This is configurable at runtime
  */
-#define RSVG_DEFAULT_DPI 90.0
-static double internal_dpi = RSVG_DEFAULT_DPI;
+#define RSVG_DEFAULT_DPI_X 90.0
+#define RSVG_DEFAULT_DPI_Y 90.0
+static double internal_dpi_x = RSVG_DEFAULT_DPI_X;
+static double internal_dpi_y = RSVG_DEFAULT_DPI_Y;
 
 static void
 rsvg_ctx_free_helper (gpointer key, gpointer value, gpointer user_data)
@@ -97,13 +99,13 @@ rsvg_start_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
 													&vbox_w, &vbox_h);
 				}
 			if ((value = rsvg_property_bag_lookup (atts, "width")))
-				width = rsvg_css_parse_normalized_length (value, ctx->dpi, vbox_w, 1);
+				width = rsvg_css_parse_normalized_length (value, ctx->dpi_x, vbox_w, 1);
 			if ((value = rsvg_property_bag_lookup (atts, "height")))
-				height = rsvg_css_parse_normalized_length (value, ctx->dpi, vbox_h, 1);
+				height = rsvg_css_parse_normalized_length (value, ctx->dpi_y, vbox_h, 1);
 			if ((value = rsvg_property_bag_lookup (atts, "x")))
-				x = rsvg_css_parse_normalized_length (value, ctx->dpi, vbox_w, 1);
+				x = rsvg_css_parse_normalized_length (value, ctx->dpi_x, vbox_w, 1);
 			if ((value = rsvg_property_bag_lookup (atts, "y")))
-				y = rsvg_css_parse_normalized_length (value, ctx->dpi, vbox_h, 1);
+				y = rsvg_css_parse_normalized_length (value, ctx->dpi_y, vbox_h, 1);
 	
 			if (has_vbox && vbox_w > 0. && vbox_h > 0.)
 				{
@@ -207,6 +209,73 @@ rsvg_start_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
 													rsvg_pixmap_destroy,
 													NULL);
 		}
+	ctx->nest_level = 1;
+	ctx->current_defs_group = NULL;
+}
+
+static void
+rsvg_start_sub_svg (RsvgHandle *ctx, RsvgPropertyBag *atts)
+{
+	int width = -1, height = -1, x = -1, y = -1;
+	double affine[6];
+	const char * id, *value;
+
+	double vbox_x = 0, vbox_y = 0, vbox_w = 0, vbox_h = 0;
+	gboolean has_vbox = FALSE;
+	id = NULL;
+
+	if (rsvg_property_bag_size (atts))
+		{
+			/* x & y should be ignored since we should always be the outermost SVG,
+			   at least for now, but i'll include them here anyway */
+			if ((value = rsvg_property_bag_lookup (atts, "viewBox")))
+				{
+					has_vbox = rsvg_css_parse_vbox (value, &vbox_x, &vbox_y,
+													&vbox_w, &vbox_h);
+				}
+			if ((value = rsvg_property_bag_lookup (atts, "width")))
+				width = rsvg_css_parse_normalized_length (value, ctx->dpi_x, ctx->width, 1);
+			if ((value = rsvg_property_bag_lookup (atts, "height")))
+				height = rsvg_css_parse_normalized_length (value, ctx->dpi_y, ctx->height, 1);
+			if ((value = rsvg_property_bag_lookup (atts, "x")))
+				x = rsvg_css_parse_normalized_length (value, ctx->dpi_x, ctx->width, 1);
+			if ((value = rsvg_property_bag_lookup (atts, "y")))
+				y = rsvg_css_parse_normalized_length (value, ctx->dpi_y, ctx->height, 1);
+			if ((value = rsvg_property_bag_lookup (atts, "id")))
+				id = value;
+
+		}
+	if (has_vbox)
+		{
+			affine[0] = width / vbox_w;
+			affine[1] = 0;
+			affine[2] = 0;
+			affine[3] = height / vbox_h;
+			affine[4] = x - vbox_x * width / vbox_w;
+			affine[5] = y - vbox_y * height / vbox_h;
+			art_affine_multiply(rsvg_state_current(ctx)->affine, affine, 
+								rsvg_state_current(ctx)->affine);	
+		}
+	else
+		{
+			affine[0] = 1;
+			affine[1] = 0;
+			affine[2] = 0;
+			affine[3] = 1;
+			affine[4] = x;
+			affine[5] = y;
+			art_affine_multiply(rsvg_state_current(ctx)->affine, affine, 
+								rsvg_state_current(ctx)->affine);				
+		}
+	rsvg_push_def_group (ctx, id);
+	ctx->nest_level++;
+}
+
+static void
+rsvg_end_sub_svg(RsvgHandle *ctx)
+{
+	if (ctx->nest_level-- > 1)
+		rsvg_pop_def_group (ctx);
 }
 
 static void
@@ -291,7 +360,7 @@ rsvg_gradient_stop_handler_start (RsvgSaxHandler *self, const xmlChar *name,
 			if ((value = rsvg_property_bag_lookup (atts, "offset")))
 				{
 					/* either a number [0,1] or a percentage */
-					offset = rsvg_css_parse_normalized_length (value, z->ctx->dpi, 1., 0.);
+					offset = rsvg_css_parse_normalized_length (value, rsvg_dpi_percentage (z->ctx), 1., 0.);
 					
 					if (offset < 0.)
 						offset = 0.;
@@ -418,19 +487,19 @@ rsvg_start_linear_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts)
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
 				id = value;
 			if ((value = rsvg_property_bag_lookup (atts, "x1"))) {
-				x1 = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
+				x1 = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state->font_size);
 				got_x1 = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "y1"))) {
-				y1 = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
+				y1 = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state->font_size);
 				got_y1 = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "x2"))) {
-				x2 = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
+				x2 = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state->font_size);
 				got_x2 = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "y2"))) {
-				y2 = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
+				y2 = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state->font_size);
 				got_y2 = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "spreadMethod")))
@@ -471,7 +540,7 @@ rsvg_start_linear_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts)
 		if (obj_bbox)
 			x2 = 1.0;
 		else
-			x2 = rsvg_css_parse_normalized_length ("100%", ctx->dpi, (gdouble)ctx->width, state->font_size);
+			x2 = rsvg_css_parse_normalized_length ("100%", ctx->dpi_x, (gdouble)ctx->width, state->font_size);
 	}
 
 	if (xlink_href != NULL)
@@ -506,6 +575,10 @@ rsvg_start_linear_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts)
 			grad->current_color = color;
 			grad->has_current_color = TRUE;
 		}
+	else
+		{
+			grad->has_current_color = FALSE;
+		}
 
 	/* gradient inherits parent/cloned information unless it's explicity gotten */
 	grad->obj_bbox = (cloned && !got_bbox) ? grad->obj_bbox : obj_bbox;
@@ -560,24 +633,24 @@ rsvg_start_radial_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts, const char *
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
 				id = value;
 			if ((value = rsvg_property_bag_lookup (atts, "cx"))) {
-				cx = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
+				cx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state->font_size);
 				got_cx = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "cy"))) {
-				cy = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
+				cy = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state->font_size);
 				got_cy = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "r"))) {
-				r = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, 
+				r = rsvg_css_parse_normalized_length (value, rsvg_dpi_percentage (ctx), 1, 
 													  state->font_size);
 				got_r = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "fx"))) {
-				fx = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
+				fx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state->font_size);
 				got_fx = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "fy"))) {
-				fy = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
+				fy = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state->font_size);
 				got_fy = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "xlink:href")))
@@ -636,19 +709,19 @@ rsvg_start_radial_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts, const char *
 		if (obj_bbox)
 			cx = 0.5;
 		else
-			cx = rsvg_css_parse_normalized_length ("50%", ctx->dpi, (gdouble)ctx->width, state->font_size);
+			cx = rsvg_css_parse_normalized_length ("50%", ctx->dpi_x, (gdouble)ctx->width, state->font_size);
 	}
 	if (!got_cy) {
 		if (obj_bbox)
 			cy = 0.5;
 		else
-			cy = rsvg_css_parse_normalized_length ("50%", ctx->dpi, (gdouble)ctx->height, state->font_size);
+			cy = rsvg_css_parse_normalized_length ("50%", ctx->dpi_y, (gdouble)ctx->height, state->font_size);
 	}
 	if (!got_r) {
 		if (obj_bbox)
 			r = 0.5;
 		else
-			r  = rsvg_css_parse_normalized_length ("50%", ctx->dpi, rsvg_viewport_percentage((gdouble)ctx->width, (gdouble)ctx->height), state->font_size);
+			r  = rsvg_css_parse_normalized_length ("50%", rsvg_dpi_percentage (ctx), rsvg_viewport_percentage((gdouble)ctx->width, (gdouble)ctx->height), state->font_size);
 	}
 	if (!got_fx) {
 		fx = cx;
@@ -670,6 +743,10 @@ rsvg_start_radial_gradient (RsvgHandle *ctx, RsvgPropertyBag *atts, const char *
 			grad->current_color = color;
 			grad->has_current_color = TRUE;
 		}
+	else
+		{
+			grad->has_current_color = FALSE;
+		}
 
 	/* gradient inherits parent/cloned information unless it's explicity gotten */
 	grad->obj_bbox = (cloned && !got_bbox) ? grad->obj_bbox : obj_bbox;
@@ -707,33 +784,39 @@ rsvg_start_pattern (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	RsvgPattern *pattern = NULL;
 	const char *id = NULL, *value;
 	double x = 0., y = 0., width = 0., height = 0.;
+	double vbx = 0., vby = 0., vbw = 1., vbh = 1.;
 	const char * xlink_href = NULL;
 	gboolean obj_bbox = TRUE;
 	gboolean obj_cbbox = FALSE;
-	gboolean got_x, got_y, got_width, got_height, got_transform, got_bbox, got_cbbox, cloned;
+	gboolean got_x, got_y, got_width, got_height, got_transform, got_bbox, got_cbbox, cloned, got_vbox;
 	double affine[6];
 	int i;
 
-	got_x = got_y = got_width = got_height = got_transform = got_bbox = got_cbbox = cloned = FALSE;
+	got_x = got_y = got_width = got_height = got_transform = got_bbox = got_cbbox = cloned = got_vbox = FALSE;
 		
 	if (rsvg_property_bag_size (atts))
 		{
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
 				id = value;
+			if ((value = rsvg_property_bag_lookup (atts, "viewBox")))
+				{
+					got_vbox = rsvg_css_parse_vbox (value, &vbx, &vby,
+													&vbw, &vbh);
+				}
 			if ((value = rsvg_property_bag_lookup (atts, "x"))) {
-				x = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
+				x = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state->font_size);
 				got_x = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "y"))) {
-				y = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
+				y = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state->font_size);
 				got_y = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "width"))) {
-				width = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
+				width = rsvg_css_parse_normalized_length (value, ctx->dpi_x, 1, state->font_size);
 				got_width = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "height"))) {
-				height = rsvg_css_parse_normalized_length (value, ctx->dpi, 1, state->font_size);
+				height = rsvg_css_parse_normalized_length (value, ctx->dpi_y, 1, state->font_size);
 				got_height = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "xlink:href")))
@@ -755,15 +838,6 @@ rsvg_start_pattern (RsvgHandle *ctx, RsvgPropertyBag *atts)
 				got_cbbox = TRUE;
 			}
 		}
-	
-	/* set up 100% as the default if not gotten */
-	
-	if (!got_width) {
-		if (obj_bbox)
-			width = 1.0;
-		else
-			width = rsvg_css_parse_normalized_length ("100%", ctx->dpi, (gdouble)ctx->width, state->font_size);
-	}
 
 	if (xlink_href != NULL)
 		{
@@ -798,10 +872,15 @@ rsvg_start_pattern (RsvgHandle *ctx, RsvgPropertyBag *atts)
 	pattern->y = (cloned && !got_y) ? pattern->y : y;
 	pattern->width = (cloned && !got_width) ? pattern->width : width;
 	pattern->height = (cloned && !got_height) ? pattern->height : height;
+	pattern->vbx = (cloned && !got_vbox) ? pattern->vbx : vbx;
+	pattern->vby = (cloned && !got_vbox) ? pattern->vby : vby;
+	pattern->vbw = (cloned && !got_vbox) ? pattern->vbw : vbw;
+	pattern->vbh = (cloned && !got_vbox) ? pattern->vbh : vbh;
+	pattern->vbox = (cloned && !got_vbox) ? pattern->vbox : got_vbox;
 
 	ctx->in_defs++;		
 
-	pattern->g = &(rsvg_push_def_group (ctx, "")->super);
+	pattern->g = &(rsvg_push_def_group (ctx, NULL)->super);
 }
 
 
@@ -978,10 +1057,6 @@ rsvg_desc_handler_end (RsvgSaxHandler *self, const xmlChar *name)
 					ctx->handler = NULL;
 				}
 		}
-	
-	/* pop the state stack */
-	ctx->n_state--;
-	rsvg_state_finalize (&ctx->state[ctx->n_state]);
 }
 
 static void
@@ -1055,10 +1130,6 @@ rsvg_title_handler_end (RsvgSaxHandler *self, const xmlChar *name)
 					ctx->handler = NULL;
 				}
 		}
-	
-	/* pop the state stack */
-	ctx->n_state--;
-	rsvg_state_finalize (&ctx->state[ctx->n_state]);
 }
 
 static void
@@ -1094,20 +1165,15 @@ rsvg_start_element (void *data, const xmlChar *name,
 		}
 	else
 		{
-			/* push the state stack */
-			if (ctx->n_state == ctx->n_state_max)
-				ctx->state = g_renew (RsvgState, ctx->state, ctx->n_state_max <<= 1);
-			if (ctx->n_state)
-				{
-					rsvg_state_inherit (&ctx->state[ctx->n_state],
-										&ctx->state[ctx->n_state - 1]);
-				}			
-			else
-				rsvg_state_init (ctx->state);
-			ctx->n_state++;
+			rsvg_state_push(ctx);
 			
 			if (!strcmp ((char *)name, "svg"))
-				rsvg_start_svg (ctx, bag);
+				{
+					if (!ctx->nest_level)
+						rsvg_start_svg (ctx, bag);
+					else
+						rsvg_start_sub_svg (ctx, bag);
+				}
 			else if (!strcmp ((char *)name, "g"))
 				rsvg_start_g (ctx, bag);
 			else if (!strcmp ((char *)name, "symbol"))
@@ -1148,7 +1214,20 @@ rsvg_start_element (void *data, const xmlChar *name,
 					ctx->in_defs++;
 					rsvg_start_mask(ctx, bag);
 				}
-			
+			else if (!strcmp ((char *)name, "clipPath"))
+				{
+					ctx->in_defs++;
+					rsvg_start_clip_path(ctx, bag);
+				}
+			else if (!strcmp ((char *)name, "marker"))
+				{
+					ctx->in_defs++;
+					rsvg_start_marker (ctx, bag);
+				}
+			else if (!strcmp ((char *)name, "switch"))
+				{
+					ctx->in_switch++;
+				}
 			/* see conicalGradient discussion above */
 			else if (!strcmp ((char *)name, "linearGradient"))
 				rsvg_start_linear_gradient (ctx, bag);
@@ -1186,6 +1265,8 @@ rsvg_end_element (void *data, const xmlChar *name)
 
 			if (!strcmp ((char *)name, "g"))
 				rsvg_end_g (ctx);
+			else if (!strcmp ((char *)name, "svg"))
+				rsvg_end_sub_svg (ctx);
 			else if (!strcmp ((char *)name, "symbol"))
 				{
 					ctx->in_defs--;
@@ -1200,13 +1281,24 @@ rsvg_end_element (void *data, const xmlChar *name)
 				rsvg_end_mask(ctx);
 				ctx->in_defs--;				
 			}
+			else if (!strcmp ((char *)name, "clipPath")){
+				rsvg_end_clip_path(ctx);
+				ctx->in_defs--;				
+			}
+			else if (!strcmp ((char *)name, "marker")){
+				rsvg_pop_def_group(ctx);
+				ctx->in_defs--;
+			}
+			else if (!strcmp ((char *)name, "switch"))
+				{
+					ctx->in_switch--;
+				}
 			else if (!strcmp ((char *)name, "pattern")) {
 				ctx->in_defs--;	
 				rsvg_pop_def_group(ctx);
 			}
-			/* pop the state stack */
-			ctx->n_state--;
-			rsvg_state_finalize (&ctx->state[ctx->n_state]);
+
+			rsvg_state_pop(ctx);
 		}
 }
 
@@ -1275,6 +1367,21 @@ static void rsvg_SAX_handler_struct_init()
     rsvgSAXHandlerStruct.endElement = rsvg_end_element;
 }
 
+void rsvg_handle_set_base_uri (RsvgHandle *handle,
+							   const char *base_uri)
+{
+	if (base_uri) {
+		if (handle->base_uri)
+			g_free (handle->base_uri);
+		handle->base_uri = g_strdup (base_uri);
+	}
+}
+
+G_CONST_RETURN char *rsvg_handle_get_base_uri (RsvgHandle *handle)
+{
+	return handle->base_uri;
+}
+
 /**
  * rsvg_error_quark
  *
@@ -1349,18 +1456,23 @@ rsvg_handle_close_impl (RsvgHandle  *handle,
 	return TRUE;
 }
 
+static void
+rsvg_state_free_func(gpointer data, gpointer user_data)
+{
+	RsvgHandle * ctx = (RsvgHandle *)user_data;
+	rsvg_state_finalize((RsvgState *)data);
+	g_mem_chunk_free(ctx->state_allocator, data);
+}
+
 void
 rsvg_handle_free_impl (RsvgHandle *handle)
 {
-	int i;
-	
 	if (handle->pango_context != NULL)
 		g_object_unref (handle->pango_context);
 	rsvg_defs_free (handle->defs);
 	
-	for (i = 0; i < handle->n_state; i++)
-		rsvg_state_finalize (&handle->state[i]);
-	g_free (handle->state);
+	g_slist_foreach(handle->state, rsvg_state_free_func, (gpointer)handle);
+	g_slist_free (handle->state);
 	
 	g_hash_table_foreach (handle->entities, rsvg_ctx_free_helper, NULL);
 	g_hash_table_destroy (handle->entities);
@@ -1377,6 +1489,11 @@ rsvg_handle_free_impl (RsvgHandle *handle)
 	if (handle->desc)
 	    	g_string_free (handle->desc, TRUE);
 
+	if (handle->base_uri)
+		g_free (handle->base_uri);
+
+	g_mem_chunk_destroy(handle->state_allocator);
+
 	g_free (handle);
 }
 
@@ -1393,7 +1510,10 @@ rsvg_handle_free_impl (RsvgHandle *handle)
  */
 G_CONST_RETURN char *rsvg_handle_get_title (RsvgHandle *handle)
 {
-	return handle->title->str;
+	if (handle->title)
+		return handle->title->str;
+	else
+		return NULL;
 }
 
 /**
@@ -1409,7 +1529,10 @@ G_CONST_RETURN char *rsvg_handle_get_title (RsvgHandle *handle)
  */
 G_CONST_RETURN char *rsvg_handle_get_desc (RsvgHandle *handle)
 {
-	return handle->desc->str;
+	if (handle->desc)
+		return handle->desc->str;
+	else
+		return NULL;
 }
 
 /**
@@ -1440,13 +1563,12 @@ rsvg_handle_new (void)
 void
 rsvg_handle_init (RsvgHandle * handle)
 {
-	handle->n_state = 0;
-	handle->n_state_max = 16;
-	handle->state = g_new (RsvgState, handle->n_state_max);
+	handle->state = NULL;
 	handle->defs = rsvg_defs_new ();
 	handle->handler_nest = 0;
 	handle->entities = g_hash_table_new (g_str_hash, g_str_equal);
-	handle->dpi = internal_dpi;
+	handle->dpi_x = internal_dpi_x;
+	handle->dpi_y = internal_dpi_y;
 	
 	handle->css_props = g_hash_table_new_full (g_str_hash, g_str_equal,
 											   g_free, g_free);
@@ -1456,6 +1578,36 @@ rsvg_handle_init (RsvgHandle * handle)
 	handle->current_defs_group = NULL;
 	handle->title = g_string_new (NULL);
 	handle->desc = g_string_new (NULL);
+
+	/* should this be G_ALLOC_ONLY? */
+	handle->state_allocator = g_mem_chunk_create (RsvgState, 256, G_ALLOC_AND_FREE);
+
+	rsvg_state_push(handle);
+}
+
+/**
+ * rsvg_set_default_dpi_x_y
+ * @dpi_x: Dots Per Inch (aka Pixels Per Inch)
+ * @dpi_y: Dots Per Inch (aka Pixels Per Inch)
+ *
+ * Sets the DPI for the all future outgoing pixbufs. Common values are
+ * 75, 90, and 300 DPI. Passing a number <= 0 to #dpi will 
+ * reset the DPI to whatever the default value happens to be.
+ *
+ * Since: 2.8
+ */
+void
+rsvg_set_default_dpi_x_y (double dpi_x, double dpi_y)
+{
+	if (dpi_x <= 0.)
+		internal_dpi_x = RSVG_DEFAULT_DPI_X;
+	else
+		internal_dpi_x = dpi_x;
+
+	if (dpi_y <= 0.)
+		internal_dpi_y = RSVG_DEFAULT_DPI_Y;
+	else
+		internal_dpi_y = dpi_y;
 }
 
 /**
@@ -1463,18 +1615,44 @@ rsvg_handle_init (RsvgHandle * handle)
  * @dpi: Dots Per Inch (aka Pixels Per Inch)
  *
  * Sets the DPI for the all future outgoing pixbufs. Common values are
- * 72, 90, and 300 DPI. Passing a number <= 0 to #dpi will 
+ * 75, 90, and 300 DPI. Passing a number <= 0 to #dpi will 
  * reset the DPI to whatever the default value happens to be.
  *
  * Since: 2.2
+ * DEPRECATED. Use rsvg_set_default_dpi_x_y() instead
  */
 void
 rsvg_set_default_dpi (double dpi)
 {
-	if (dpi <= 0.)
-		internal_dpi = RSVG_DEFAULT_DPI;
-	else
-		internal_dpi = dpi;
+	rsvg_set_default_dpi_x_y (dpi, dpi);
+}
+
+/**
+ * rsvg_handle_set_dpi_x_y
+ * @handle: An #RsvgHandle
+ * @dpi_x: Dots Per Inch (aka Pixels Per Inch)
+ * @dpi_y: Dots Per Inch (aka Pixels Per Inch)
+ *
+ * Sets the DPI for the outgoing pixbuf. Common values are
+ * 75, 90, and 300 DPI. Passing a number <= 0 to #dpi will 
+ * reset the DPI to whatever the default value happens to be.
+ *
+ * Since: 2.8
+ */
+void
+rsvg_handle_set_dpi_x_y (RsvgHandle * handle, double dpi_x, double dpi_y)
+{
+	g_return_if_fail (handle != NULL);
+	
+    if (dpi_x <= 0.)
+        handle->dpi_x = internal_dpi_x;
+    else
+        handle->dpi_x = dpi_x;
+	
+	if (dpi_y <= 0.)
+        handle->dpi_y = internal_dpi_y;
+    else
+        handle->dpi_y = dpi_y;
 }
 
 /**
@@ -1483,20 +1661,16 @@ rsvg_set_default_dpi (double dpi)
  * @dpi: Dots Per Inch (aka Pixels Per Inch)
  *
  * Sets the DPI for the outgoing pixbuf. Common values are
- * 72, 90, and 300 DPI. Passing a number <= 0 to #dpi will 
+ * 75, 90, and 300 DPI. Passing a number <= 0 to #dpi will 
  * reset the DPI to whatever the default value happens to be.
  *
  * Since: 2.2
+ * DEPRECATED. Use rsvg_handle_set_dpi_x_y() instead
  */
 void
 rsvg_handle_set_dpi (RsvgHandle * handle, double dpi)
 {
-	g_return_if_fail (handle != NULL);
-	
-    if (dpi <= 0.)
-        handle->dpi = internal_dpi;
-    else
-        handle->dpi = dpi;
+	rsvg_handle_set_dpi_x_y (handle, dpi, dpi);
 }
 
 /**
